import numpy.testing as npt
import pandas as pd
import os
# from quant.multiquant_parser import process_input_data
from quant.table_processing import *
from test_input.constants import *

std_df = pd.DataFrame({'Original Filename': [1, 1, 2, 2, 3, 4, 1, 2, 3, 4],
                       'Sample Name': ['a', 'a', 'b', 'b', 'a', 'b', 'a', 'b',
                                       'a', 'b'],
                       'Area': [5, 8, 10, 12, 15, 20, 25, 30, 35, 40],
                       'Unlabeled Fragment': ['P', 'P', 'P', 'P', 'P', 'P', 'Q',
                                              'Q', 'Q', 'Q'],
                       'Concentration': [1, 1, 2, 2, 1, 2, 1, 2, 1, 2],
                       'Unit': ['M', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'M',
                                'M']})

metab_table = std_df[std_df[constants.PARENT] == 'P']


def test__compute_sum_by_fragments():
    std = TableProcessing(std_df)
    std_table = std._compute_sum_by_fragments(metab_table)
    expected_std_table = pd.DataFrame({'Original Filename': [1, 2, 3, 4],
                                       'Area': [13, 22, 15, 20],
                                       'Unlabeled Fragment': ['P', 'P', 'P',
                                                              'P'],
                                       'Concentration': [1, 2, 1, 2],
                                       'Unit': ['M', 'M', 'M', 'M']})
    expected_std_table = expected_std_table[
        ['Concentration', 'Unit', 'Unlabeled Fragment',
         'Original Filename', 'Area']]
    npt.assert_array_equal(std_table, expected_std_table)


def test_get_standard_table_no_back_corr():
    std = TableProcessing(std_df)
    std_table = std.get_standard_table('P', False)
    expected_std_table = pd.DataFrame(
        {'Area_0': [13, 22], 'Area_1': [15, 20], 'Std_dev': [1.41, 1.41],
         'Concentration': [1, 2], 'Unit': ['M', 'M'],
         'Unlabeled Fragment': ['P', 'P'],
         'Average_Area': [14.0, 21.0]})
    expected_std_table = expected_std_table[
        ['Area_0', 'Area_1', 'Std_dev', 'Concentration', 'Unit',
         'Unlabeled Fragment', 'Average_Area']]
    npt.assert_array_equal(std_table, expected_std_table)


def test_get_standard_table_back_corr():
    std = TableProcessing(std_df)
    std_table = std.get_standard_table('P', True)
    expected_std_table = pd.DataFrame(
        {'Area_0': [0, 9], 'Area_1': [0, 5], 'Std_dev': [0.0, 2.83],
         'Concentration': [1, 2], 'Unit': ['M', 'M'],
         'Unlabeled Fragment': ['P', 'P'],
         'Average_Area': [0.0, 7.0]})
    expected_std_table = expected_std_table[
        ['Area_0', 'Area_1', 'Std_dev', 'Concentration', 'Unit',
         'Unlabeled Fragment', 'Average_Area']]
    npt.assert_array_equal(std_table, expected_std_table)


def test_get_all_std_tables_df():
    std = TableProcessing(std_df)
    standard_df, unit = std.get_all_std_tables_df(False)
    expected_standard_df = pd.DataFrame(
        {'Area_0': [13, 22, 25, 30], 'Area_1': [15, 20, 35, 40],
         'Std_dev': [1.41, 1.41, 7.07, 7.07],
         'Concentration': [1, 2, 1, 2],
         'Unit': ['M', 'M', 'M', 'M'],
         'Unlabeled Fragment': ['P', 'P', 'Q', 'Q'],
         'Average_Area': [14.0, 21.0, 30.0, 35.0]})
    expected_standard_df = expected_standard_df[
        ['Area_0', 'Area_1', 'Std_dev', 'Concentration', 'Unit',
         'Unlabeled Fragment', 'Average_Area']]
    assert all([np.array_equal(standard_df, expected_standard_df), unit == 'M'])

